/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_MICROSECONDS_STAMP_HH_
# define TOOL_MICROSECONDS_STAMP_HH_

# include <wipal/tool/microseconds.hh>
# include <wipal/tool/hash.hh>

namespace tool
{

  /**
   * Mixin class to add a %microsecond count to an existing class.
   *
   * This class subclasses its template parameter.  Thus, it behaves
   * mostly like it.  This class adds a microsecond count attribute,
   * which is initialized through the constructor, and which you may
   * access with the microsecond() methods.
   *
   * By default, this class' == operator does not compare the
   * microsecond count.  Subclass it and provide a custom == operator
   * if this is not the desired behavior.
   */
  template <class T>
  struct microseconds_stamp: public T
  {
    /// Construct from an original @c T object and a microsecond count.
    microseconds_stamp(const tool::microseconds&, const T& = T ());

    /// @name Access to the microsecond count
    //@{
    const tool::microseconds&	microseconds() const;
    tool::microseconds&		microseconds();
    //@}

    /**
     * @name Equality checks
     *
     * @warning This method compares the super type, but @b not the
     *		microsecond count.
     */
    //@{
    bool	operator == (const microseconds_stamp& rhs) const;
    bool	operator != (const microseconds_stamp& rhs) const;
    //@}

  private:
    tool::microseconds		microseconds_;
  };

} // End of namespace tool.

WP_HASH_NAMESPACE_BEGIN

  template <class T>
  struct hash< tool::microseconds_stamp<T> >: public hash<T>
  {
    size_t	operator() (const tool::microseconds_stamp<T>& s) const;
  };

WP_HASH_NAMESPACE_END

# include "microseconds_stamp.hxx"

#endif // TOOL_MICROSECONDS_STAMP_HH_
