/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_SEQCTL_HH_
# define WIFI_FRAME_SEQCTL_HH_

extern "C"
{
# include <inttypes.h>
}

# include <string>
# include <ostream>

namespace wifi
{

  namespace frame
  {

    /// IEEE 802.11 Sequence number.
    struct seqnum
    {
      seqnum(unsigned value = 0);

      unsigned	get() const;

      bool	operator <  (const seqnum& rhs) const;
      bool	operator <= (const seqnum& rhs) const;
      bool	operator >  (const seqnum& rhs) const;
      bool	operator >= (const seqnum& rhs) const;
      bool	operator == (const seqnum& rhs) const;
      bool	operator != (const seqnum& rhs) const;

      int	operator - (const seqnum& rhs) const;

    private:
      unsigned	v_;
    };

    /// IEEE 802.11 Sequence control field.
    struct seqctl
    {
      seqctl(unsigned seqnum = 0, unsigned fragnum = 0);
      seqctl(const uint16_t raw);
      seqctl(const void* raw);

      seqnum	seqnum_get() const;
      unsigned	fragnum_get() const;

      bool	operator <  (const seqctl& rhs) const;
      bool	operator <= (const seqctl& rhs) const;
      bool	operator >  (const seqctl& rhs) const;
      bool	operator >= (const seqctl& rhs) const;
      bool	operator == (const seqctl& rhs) const;
      bool	operator != (const seqctl& rhs) const;

    private:
      uint16_t	seqctl_;
    };

    /// Dump an @link seqctl sequence control field @endlink in human
    /// readable notation.
    std::ostream&
    operator << (std::ostream& os, const seqctl& a);

    /// Make a human readable string from an @link seqctl sequence
    /// control field @endlink .
    std::string
    make_string(const seqctl& a);

  } // End of namespace wifi::frame.

} // End of namespace wifi.

# include "seqctl.hxx"

#endif // ! WIFI_FRAME_SEQCTL_HH_
