#! /usr/bin/env python

import scapy
import random
import sys

PrismHeader     = scapy.PrismHeader
Dot11           = scapy.Dot11
Dot11Beacon     = scapy.Dot11Beacon

src="42:42:42:42:42:42"
dst="d0:d0:d0:d0:d0:d0"
bss="fa:be:cc:51:be:ef"

def ph(time, noise = 0):
    return PrismHeader(mactime = time, hosttime = time / 10, noise = noise)

def mgt(s, a1, a2, a3):
    return Dot11(SC = s << 4, addr1 = a1, addr2 = a2, addr3 = a3)

def beacon(s):
    return mgt(s, "ff:ff:ff:ff:ff:ff", bss, bss) / \
        Dot11Beacon(timestamp = random.randint(0, 2 ** 64 - 1))

def data(s):
    return Dot11(type = 2, subtype = 4, SC = s << 4,
                 addr1 = dst, addr2 = src, addr3 = bss)

def ack():
    return Dot11(type = 1, subtype = 13, addr1 = src)

b1 = ph(1000) / beacon(0)
d1 = ph(2000, 42) / data(0)
a1 = ph(3000, 51) / ack()
d2 = ph(4000, 60) / data(1)
a2 = ph(5000) / ack()
a3 = ph(6000) / ack()
b2 = ph(7000) / beacon(1)

nonoise = [b1,             a2, a3, b2]
all     = [b1, d1, a1, d2, a2, a3, b2]
lhs     = [b1, d1, a1, d2, a2,     b2]
rhs     = [b1,     a1,         a3, b2]

def shift(l, a, b, j):
    for i in range(len(l)):
        l[i] = l[i].copy()
        l[i].mactime = int (a * l[i].mactime + b + random.randint(0, j))
        l[i].hosttime = l[i].mactime / 10

# Avoid huge clock variations.
a = 0
while a < 0.5:
    a = random.random()

shift(lhs, a, random.randint(0, 2 ** 32 - 1), 20)

scapy.wrpcap(sys.argv[1] + "-nonoise.pcap",     nonoise, linktype = 119)
scapy.wrpcap(sys.argv[1] + "-union.pcap",       all,     linktype = 119)
scapy.wrpcap(sys.argv[1] + "-lhs.pcap",         lhs,     linktype = 119)
scapy.wrpcap(sys.argv[1] + "-rhs.pcap",         rhs,     linktype = 119)
